# Table.py -- Pamhyr
# Copyright (C) 2023-2024  INRAE
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

# -*- coding: utf-8 -*-

import logging
import traceback
from tools import trace, timer

from PyQt5.QtCore import (
    Qt, QVariant, QAbstractTableModel,
    QCoreApplication, QModelIndex, pyqtSlot,
    QRect,
)

from PyQt5.QtWidgets import (
    QDialogButtonBox, QPushButton, QLineEdit,
    QFileDialog, QTableView, QAbstractItemView,
    QUndoStack, QShortcut, QAction, QItemDelegate,
    QComboBox,
)

from View.Tools.PamhyrTable import PamhyrTableModel

from View.InitialConditionsAdisTS.UndoCommand import (
    SetCommand,
)

logger = logging.getLogger()

_translate = QCoreApplication.translate

class InitialConditionTableDefaultModel(PamhyrTableModel):
    def __init__(self, **kwargs):
        super(InitialConditionTableDefaultModel, self).__init__(**kwargs)

    def data(self, index, role):
        if role != Qt.ItemDataRole.DisplayRole:
            return QVariant()

        row = index.row()
        column = index.column()

        if self._headers[column] is "name":
            return self._data[row].name
        elif self._headers[column] is "concentration":
            n = self._data[row].concentration
            if n is None:
                return self._trad['not_associated']
            return n
        elif self._headers[column] is "eg":
            n = self._data[row].eg
            if n is None:
                return self._trad['not_associated']
            return n
        elif self._headers[column] is "em":
            n = self._data[row].em
            if n is None:
                return self._trad['not_associated']
            return n
        elif self._headers[column] is "ed":
            n = self._data[row].ed
            if n is None:
                return self._trad['not_associated']
            return n

        return QVariant()

    def setData(self, index, value, role=Qt.EditRole):
        if not index.isValid() or role != Qt.EditRole:
            return False

        row = index.row()
        column = index.column()

        try:
            if self._headers[column] is not None:
                self._undo.push(
                    SetCommand(
                        self._data, row, self._headers[column], value
                    )
                )
        except Exception as e:
            logger.info(e)
            logger.debug(traceback.format_exc())

        self.dataChanged.emit(index, index)
        return True

    def undo(self):
        self._undo.undo()
        self.layoutChanged.emit()

    def redo(self):
        self._undo.redo()
        self.layoutChanged.emit()

